
// LoaderCanvas.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, November 2004

/* Create a 3D scene in retained mode (i.e. build a scenegraph).

   The scene has a floor, blue background, a mobile camera, and 
   a loaded model.

   The model comes from a M3G file, and can be loaded with a 
   specified scale and (x,y,z) translation. The model may be
   at the root of the M3G file or just below the root.

   The scene is updated and repainted by a timer task in 
   LoaderM3G which calls update() periodically.
*/

import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class LoaderCanvas extends Canvas implements CommandListener 
{
  private Command exitCmd; 

  private Graphics3D iG3D;

  private LoaderM3G top;
  private World scene;

  // scene elements
  private MobileCamera mobCam;
  private Group cameraGroup;


  public LoaderCanvas(LoaderM3G top) 
  { 
    this.top = top;

    exitCmd = new Command("Exit", Command.EXIT, 0);
    addCommand(exitCmd);
    setCommandListener(this);

    iG3D = Graphics3D.getInstance();
    scene = new World();   // create scene graph
    buildScene();
  }  // end of LoaderCanvas()


  public void commandAction(Command command, Displayable disp)
  { 
    if (command == exitCmd)    // finish the morphing
      top.finishGame();
    else   // execution should not get here
      System.out.println("Command Action Error: " + command);
  }


  private void buildScene()
  // add elements to the scene
  {
    addCamera();
    addLights();
    addBackground();
    addModel();
    addFloor();
  } // end of buildScene()


  private void addCamera()
  {
    mobCam = new MobileCamera( getWidth(), getHeight());

    scene.addChild( mobCam.getCameraGroup() );
    scene.setActiveCamera( mobCam.getCamera() ); 
  }  // end of addCamera()


  private void addLights()
  {
    Light light = new Light();  // default white, directional light
    light.setIntensity(1.25f);  // make it a bit brighter
    light.setOrientation(-45.0f, 1.0f, 0, 0); // pointing down and into scene
                  // -45
    scene.addChild(light);
  }  // end of addLights()


  private void addBackground()
  { Background backGnd = new Background();
    backGnd.setColor(0x00bffe); // a light blue background
    scene.setBackground(backGnd);
  }

  private void addFloor()
  {
    Image2D floorIm = loadImage("/bigGrid.gif");   // large, so slow to load
    Floor f = new Floor( floorIm, 8);   // 8 by 8 size
    scene.addChild( f.getFloorMesh() );
  }  // end of addFloor()


  private Image2D loadImage(String fn)
  // load the image stored in fn
  { Image2D im = null;
    try {
       im = (Image2D)Loader.load(fn)[0];
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()




  // ------------ process key presses and releases ------------


  protected void keyPressed(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.pressedKey(gameAction);
  }

  protected void keyReleased(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.releasedKey(gameAction);
  }

  // ------------------ update and paint the canvas ------------------

  public void update()
  // called by the timer task in LoaderM3G
  {
    mobCam.update();
    repaint();
  }  // end of update()


  protected void paint(Graphics g)
  {
    iG3D.bindTarget(g);
    try {
      iG3D.render(scene);
    }
    catch(Exception e)
    { e.printStackTrace();  }
    finally {
      iG3D.releaseTarget();
    }

    // show the camera's details
    g.drawString( mobCam.getKeyMode(), 5,5, Graphics.TOP|Graphics.LEFT);
    g.drawString( mobCam.getPosition(), 5,18, Graphics.TOP|Graphics.LEFT);
    g.drawString( mobCam.getRotation(), 5,31, Graphics.TOP|Graphics.LEFT);
  } // end of paint()


  // ---------------- load a model --------------------


  private void addModel()
  /* The data passed to loadRootModel() and loadSceneModel() come from
     manipulating the models in ViewM3G.
  */
  {
    Group modelGroup = null;

    // example from Leon HongKong
    // modelGroup = loadRootModel("/tube.m3g", 0.0048f, 0.4f, 0.1f, 0); 
    // Material m = (Material) modelGroup.find(5);
    // m.setColor(Material.DIFFUSE, 0xFFC3C3);   // pink
        // colour obtained using ColorCalc (http://labrocca.com/colorcalc/)

    // examples from WTK 2.2
    // modelGroup = loadSceneModel("/pogoroo.m3g", 421721094, 1.0f, 0, 0, 0); 
    // modelGroup = loadSceneModel("/skaterboy.m3g", 421721094, 0.2897f, -0.19f, 0, 0); 

    // examples from Nokia "3-D Game Development on JSR-184"
    modelGroup = loadSceneModel("/pond_vilkutus2.m3g", 421721094, 1.0f, 0, 0, 0); 
    // modelGroup = loadSceneModel("/otokka_jump2.m3g", 421721094, 0.424f, 0, 0, 0); 
    // modelGroup = loadSceneModel("/nokia_on_ice.m3g", 421721094, 0.1635f, 0, 1.1f, 0); 
    // modelGroup = loadSceneModel("/tunnel.m3g", 421721094, 0.0431f, 0, 0, 0); 

    scene.addChild( modelGroup );  // add the model

 }  // end of addModel()


  private Group loadRootModel(String fn, float scale,
                                float xMove, float yMove, float zMove)
  /* Load the model that's at the root of the scene in the M3G file fn.
     Scale and reposition the model. */
  { 
    System.out.println("Loading model from " + fn);

    Object3D[] parts = null;
    try {
      parts = Loader.load(fn);
    } 
    catch (Exception e) 
    {  e.printStackTrace();  }

    Group mGroup = (Group) parts[0];   // use Group node at the root
    mGroup.setTranslation(xMove, yMove, zMove);
    mGroup.scale(scale, scale, scale);
          // translations and scale obtained from ViewM3G
    return mGroup;
  }  // end of loadRootModel()



  private Group loadSceneModel(String fn, int id, float scale,
                                float xMove, float yMove, float zMove)
  /* Load the model that's below the root of the scene in the M3G file fn.
     This is usually because the root is a World node.
     Scale and reposition the model. 

     Most models that are in a World scene have been created with 3DS Max,
     and so are facing downwards. Rotate them up to face front, and lift
     them up a little.
  */
  { 
    System.out.println("Loading model from " + fn);

    Object3D[] parts = null;
    try {
      parts = Loader.load(fn);
    } 
    catch (Exception e) 
    {  e.printStackTrace();  }

    Group root = (Group) parts[0];
    Group mGroup = (Group) root.find(id);   // use Group with user ID id

    root.removeChild(mGroup);  
         // we're assuming that mGroup is directly below the root

    // reposition the non-root model
    // translations and scaling obtained from ViewM3G
    Group posGroup = new Group();
    posGroup.setOrientation(-90.0f, 1.0f, 0, 0);  // rotate -90 around x-axis
    posGroup.setTranslation(xMove, yMove+0.1f, zMove);  
          // translation + small lift
    posGroup.scale(scale, scale, scale);
    posGroup.addChild(mGroup);

    return posGroup;
  }  // end of loadSceneModel()


}  // end of LoaderCanvas class
