
// MobileModelCamera.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, October 2004

/* MobileModelCamera is an extension of MobileCamera, so it
   can move, rotate, and 'float' the camera as before. It
   can also translate and scale the supplied Group node.

   This means there are now _five_ modes:
          * MOVE, ROTATE, and FLOAT as before, and
          * SCALE_YTRANS, XZTRANS for group scaling and translation

   The idea is that the Group node will be the top-level of a
   model loaded from an M3G file, which needs to be scaled and
   translated to make it suitable for a J2ME application.

   The scale factor and translations are reported to the screen,
   so can be utilized in the other application using:
      modelGroup.setTranslation(xMove, yMove, zMove);
      modelGroup.scale(scaleFactor, scaleFactor, scaleFactor);
*/

import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class MobileModelCamera
{
  // initial x-axis rotation and position for the camera
  private static final float X_ANGLE = -10.0f;  // degrees
  private static final float X_POS = 0.0f; 
  private static final float Y_POS = 0.5f; 
  private static final float Z_POS = 2.0f; 
    // move back and up, rotate down

  // translation and rotation increments for the camera
  private static final float ANGLE_INCR = 10.0f;   // degrees
  private static final float MOVE_INCR = 0.1f;   // also used for the model

  // scaling increments for the model
  private static final float SCALE_UP = 1.1f;
  private static final float SCALE_DOWN = 1.0f/SCALE_UP;

  // key mode constants
  private static final int MOVE = 0;    // move left, right, fwd, back
  private static final int ROTATE = 1;  // turn left, right, up, down
  private static final int FLOAT = 2;   // move up, down
  private static final int SCALE_YTRANS = 3;   // model scale, y-trans
  private static final int XZTRANS = 4;        // model x- or z- trans
  private static final int NUM_MODES = 5;

  // booleans for remembering key presses
  private boolean upPressed = false;
  private boolean downPressed = false;
  private boolean leftPressed = false;
  private boolean rightPressed = false;

  private int keyMode = MOVE;   // current key mode

  // for examining the camera's (x,y,z) position
  private Transform trans = new Transform();
  private float transMat[] = new float[16];
  private float xCoord, yCoord, zCoord;

  // for storing the camera's current x- and y- rotations
  private float xAngle, yAngle;

  // scene graph elements for the camera
  private Group transGroup;
  private Camera cam;

  // model manipulation 
  private Group modelGroup;  // Group node for the model
  private float scaleFactor;
  private float xMove, yMove, zMove;


  public MobileModelCamera(Group mg, int width, int height)
  {
    modelGroup = mg;
    scaleFactor = 1.0f;
    xMove = 0.0f;  yMove = 0.0f; zMove = 0.0f;

    cam = new Camera();
    float aspectRatio = ((float) width) / ((float) height);
    cam.setPerspective(70.0f, aspectRatio, 0.1f, 50.0f);
    cam.postRotate(X_ANGLE, 1.0f, 0, 0);  // apply x-axis rotations to camera

    // translation an y-axis rotation group for the model
    transGroup = new Group();      // no initial rotation
    trans.postTranslate(X_POS, Y_POS, Z_POS);
    transGroup.setTransform(trans);
    transGroup.addChild(cam);

    // store initial rotations
    xAngle = X_ANGLE; yAngle = 0.0f;
  }  // end of MobileModelCamera()


  public Group getCameraGroup()
  {  return transGroup; }

  public Camera getCamera()
  {  return cam; }


  // ----- handle key presses / releases, and return details ------

  public void pressedKey(int gameAction)
  {
    switch(gameAction) {
      case Canvas.UP:    upPressed = true;    break;
      case Canvas.DOWN:  downPressed = true;  break;
      case Canvas.LEFT:  leftPressed = true;  break;
      case Canvas.RIGHT: rightPressed = true; break;
      case Canvas.FIRE: keyMode = (keyMode + 1) % NUM_MODES;  break;
      default: break;
    }
  }  // end of pressedKey()


  public void releasedKey(int gameAction)
  {
    switch(gameAction) {
      case Canvas.UP:    upPressed = false;    break;
      case Canvas.DOWN:  downPressed = false;  break;
      case Canvas.LEFT:  leftPressed = false;  break;
      case Canvas.RIGHT: rightPressed = false; break;
      default: break;
    }
  }  // end of releasedKey()


  public String getKeyMode()
  {
    switch(keyMode) {
      case MOVE: return "Mode: move";
      case ROTATE: return "Mode: rotate";
      case FLOAT: return "Mode: float";
      case SCALE_YTRANS: return "Mode: scale/y-";
      case XZTRANS: return "Mode: x-/z-";
      default: return "Mode: ??";
    }
  } // end of getKeyMode()


  // ------------ access camera manipulation info. ----------------

  public String getPosition()
  {  
    storePosition();
    return "Pos: (" + round2dp(xCoord) + ", " + 
                      round2dp(yCoord) + ", " + 
                      round2dp(zCoord) + ")";  
  }  // end of getPosition()


  private float round2dp(float num)
  {  return ((int)((num+0.005)*100.0f))/100.0f;  }


  private void storePosition()
  // extract the current (x,y,z) position from transGroup
  {
    transGroup.getTransform(trans);
    trans.get(transMat);
    xCoord = transMat[3];
    yCoord = transMat[7];
    zCoord = transMat[11]; 
  }  // end of storePosition()


  public String getRotation()
  {  return "Rot: (" + xAngle + ", " + yAngle + ", 0.0)";  }


  // ----------- access model manipulation info. -----------------


  public String getModelScale()
  {  return "Scale: " + scaleFactor;  }


  public String getModelMove()
  {  return "Move: (" + round2dp(xMove) + ", " + 
                      round2dp(yMove) + ", " + 
                      round2dp(zMove) + ")";  
  }  // end of getModelMove()



  // ------------- update camera position and rotation ---------------

  public void update()
  {
    switch(keyMode) {
      case MOVE: updateMove();  break;
      case ROTATE: updateRotation();  break;
      case FLOAT: updateFloating();  break;
      case SCALE_YTRANS: scaleTransModel(); break;
      case XZTRANS: transModel(); break;
      default: break;
    }
  }  // end of update()


  private void updateMove()
  {
    transGroup.getTransform(trans);
    if (upPressed)          // move forward
      trans.postTranslate(0, 0, -MOVE_INCR);
    else if (downPressed)   // move backwards
      trans.postTranslate(0, 0, MOVE_INCR);
    else if (leftPressed)   // move to the left
      trans.postTranslate(-MOVE_INCR, 0, 0);
    else if (rightPressed)  // move to the right
      trans.postTranslate(MOVE_INCR, 0, 0);
    transGroup.setTransform(trans);
  }  // end of updateMove()


  private void updateRotation()
  {
    if (upPressed) {  // rotate up around x-axis
      cam.postRotate(ANGLE_INCR, 1.0f, 0, 0);
      xAngle += ANGLE_INCR;
    }
    else if (downPressed) {  // rotate down around x-axis
      cam.postRotate(-ANGLE_INCR, 1.0f, 0, 0);
      xAngle -= ANGLE_INCR;
    }
    else if (leftPressed)   // rotate left around the y-axis
      yAngle += ANGLE_INCR;
    else if (rightPressed)   // rotate right around the y-axis
      yAngle -= ANGLE_INCR;

    // angle values are modulo 360 degrees
    if (xAngle >= 360.0f)
      xAngle -= 360.0f;
    else if (xAngle <= -360.0f)
      xAngle += 360.0f;

    if (yAngle >= 360.0f)
      yAngle -= 360.0f;
    else if (yAngle <= -360.0f)
      yAngle += 360.0f;

    // apply the y-axis rotation to transGroup
    if (leftPressed || rightPressed) {
      storePosition();
      trans.setIdentity();
      trans.postTranslate(xCoord, yCoord, zCoord);
      trans.postRotate(yAngle, 0, 1.0f, 0);
      transGroup.setTransform(trans);
    }
  }  // end of updateRotation()


  private void updateFloating()
  {
    transGroup.getTransform(trans);
    if (upPressed)          // move up
      trans.postTranslate(0, MOVE_INCR, 0);
    else if (downPressed)   // move down
      trans.postTranslate(0, -MOVE_INCR, 0);
    else if (leftPressed)   // move to the left (same as updateMove())
      trans.postTranslate(-MOVE_INCR, 0, 0);
    else if (rightPressed)   // move to the right (same as updateMove())
      trans.postTranslate(MOVE_INCR, 0, 0);
    transGroup.setTransform(trans);
  }  // end of updateFloating()



  // --------------- model manipulation -------------------


  private void scaleTransModel()
  // scale or y-translate the model
  {
    if (upPressed) {          // scale up
      modelGroup.scale(SCALE_UP, SCALE_UP, SCALE_UP);
      scaleFactor *= SCALE_UP;
    }
    else if (downPressed) {   // scale down
      modelGroup.scale(SCALE_DOWN, SCALE_DOWN, SCALE_DOWN);
      scaleFactor *= SCALE_DOWN;
    }
    else if (leftPressed) {   // move down the y-axis
      modelGroup.translate(0, -MOVE_INCR, 0);
      yMove -= MOVE_INCR;
    }
    else if (rightPressed) {   // move up the y-axis
      modelGroup.translate(0, MOVE_INCR, 0);
      yMove += MOVE_INCR;
    }
  }  // end of scaleTransModel()



  private void transModel()
  // x- or z- translate the model
  {
    if (upPressed) {          // move back on z-axis
      modelGroup.translate(0, 0, -MOVE_INCR);
      zMove -= MOVE_INCR;
    }
    else if (downPressed) {   // scale forward on z-axis
      modelGroup.translate(0, 0, MOVE_INCR);
      zMove += MOVE_INCR;
    }
    else if (leftPressed) {   // move left on x-axis
      modelGroup.translate(-MOVE_INCR, 0, 0);
      xMove -= MOVE_INCR;
    }
    else if (rightPressed) {   // move right on x-axis
      modelGroup.translate(MOVE_INCR, 0, 0);
      xMove += MOVE_INCR;
    }
  }  // end of transModel()


} // end of MobileModelCamera class
