
// ViewCanvas.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, November 2004

/* Create a 3D scene in retained mode (i.e. build a scenegraph).

   The scene has a floor, blue background, and a mobile camera.
   The model comes from a M3G file, and its size and position
   can be adjusted. Any scaling and moves are written to the
   screen.

   The scene is updated and repainted by a timer task in 
   ViewM3G which calls update() periodically.
*/

import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;


public class ViewCanvas extends Canvas implements CommandListener 
{
  // example from Leon HongKong
  // private static final String MODEL_FN = "/tube.m3g";   // M3G file to load

  // examples from WTK 2.2
  // private static final String MODEL_FN = "/pogoroo.m3g"; 
  // private static final String MODEL_FN = "/skaterboy.m3g"; 

  // examples from Nokia "3-D Game Development on JSR-184"
  private static final String MODEL_FN = "/pond_vilkutus2.m3g"; 
  // private static final String MODEL_FN = "/otokka_jump2.m3g"; 
  // private static final String MODEL_FN = "/nokia_on_ice.m3g"; 
  // private static final String MODEL_FN = "/tunnel.m3g"; 


  private Command exitCmd; 

  private Graphics3D iG3D;

  private ViewM3G top;
  private World scene;

  // scene elements
  private MobileModelCamera mobCam;
  private Group cameraGroup;


  public ViewCanvas(ViewM3G top) 
  { 
    this.top = top;

    exitCmd = new Command("Exit", Command.EXIT, 0);
    addCommand(exitCmd);
    setCommandListener(this);

    iG3D = Graphics3D.getInstance();
    scene = new World();   // create scene graph
    buildScene();
  }  // end of ViewCanvas()


  public void commandAction(Command command, Displayable disp)
  { 
    if (command == exitCmd)    // finish the morphing
      top.finishGame();
    else   // execution should not get here
      System.out.println("Command Action Error: " + command);
  }


  private void buildScene()
  // add elements to the scene
  {
    ViewModel vm = new ViewModel(MODEL_FN);
    Group modelGroup = vm.getModel();
    scene.addChild( modelGroup );  // add the model

    addCamera(modelGroup);
    addLights();
    addBackground();
    addFloor();
  } // end of buildScene()


  private void addCamera(Group modelGroup)
  {
    mobCam = new MobileModelCamera(modelGroup, getWidth(), getHeight());
         // the mobile camera can affect the model

    scene.addChild( mobCam.getCameraGroup() );
    scene.setActiveCamera( mobCam.getCamera() ); 
  }  // end of addCamera()


  private void addLights()
  {
    Light light = new Light();  // default white, directional light
    light.setIntensity(1.25f);  // make it a bit brighter
    light.setOrientation(-45.0f, 1.0f, 0, 0); // pointing down and into scene
                  // -45
    scene.addChild(light);
  }  // end of addLights()


  private void addBackground()
  { Background backGnd = new Background();
    backGnd.setColor(0x00bffe); // a light blue background
    scene.setBackground(backGnd);
  }

  private void addFloor()
  { Image2D floorIm = loadImage("/bigGrid.gif");
    Floor f = new Floor( floorIm, 8);   // 8 by 8 size
    scene.addChild( f.getFloorMesh() );
  }  // end of addFloor()


  private Image2D loadImage(String fn)
  // load the image stored in fn
  { Image2D im = null;
    try {
       im = (Image2D)Loader.load(fn)[0];
    }
    catch (Exception e)
    { System.out.println("Cannot make image from " + fn); }
    return im;
  }  // end of loadImage()



  // ------------ process key presses and releases ------------


  protected void keyPressed(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.pressedKey(gameAction);
  }

  protected void keyReleased(int keyCode)
  { int gameAction = getGameAction(keyCode);
    mobCam.releasedKey(gameAction);
  }

  // ------------------ update and paint the canvas ------------------

  public void update()
  // called by the timer task in ViewM3G
  {
    mobCam.update();
    repaint();
  }  // end of update()


  protected void paint(Graphics g)
  {
    iG3D.bindTarget(g);
    try {
      iG3D.render(scene);
    }
    catch(Exception e)
    { e.printStackTrace();  }
    finally {
      iG3D.releaseTarget();
    }

    // show the camera's details
    g.drawString( mobCam.getKeyMode(), 5,5, Graphics.TOP|Graphics.LEFT);
    g.drawString( mobCam.getPosition(), 5,18, Graphics.TOP|Graphics.LEFT);
    g.drawString( mobCam.getRotation(), 5,31, Graphics.TOP|Graphics.LEFT);

    // show the model's details on the right of the screen
    int rightSide = getWidth() - 5;
    g.drawString( mobCam.getModelScale(), rightSide,18, Graphics.TOP|Graphics.RIGHT);
    g.drawString( mobCam.getModelMove(), rightSide,31, Graphics.TOP|Graphics.RIGHT);

  } // end of paint()


}  // end of ViewCanvas class
