
// ViewModel.java
// Andrew Davison, dandrew@fivedots.coe.psu.ac.th, November 2004

/* Load the M3G model stored in fn. The model can be part of
   a complete scene (with a World node at its root), or an 
   exported model (with a Group node at its root).

   modelGroup is the first Group node found during a top-down
   search (traversal) of the scene returned by Loader.load(). 

   If modelGroup is not the root of the scene, then it is detached
   from its parent (probably a World node) so it can be added to
   the ViewM3G scene.

   The detached model is also rotated around the x-axis and lifted
   up the y-axis slightly, since it is probably an exported 3DS Max
   scene where the positive z-axis is pointing downwards.

   If showExamination is true, then the class name of each visited
   node, and its user ID are printed to stdout. 

   Extra information is displayed when Group, VertexBuffer, and
   Material nodes are encountered (this part of the software could
   certainly be extended).
*/


import javax.microedition.m3g.*;


public class ViewModel 
{
  private Group modelGroup;     // top-level Group node for the model

  private boolean foundGroup;   // Group node found during the search?
  private boolean isSceneRoot;  // is found Group node the root of the scene graph?

  private boolean showExamination = false;    
     /** CHANGE THIS TO SEE, OR NOT SEE, THE EXAMINATION **/

  // for looking at the transform in a Group node
  private float trans[] = new float[16];
  private Transform modelTransform = new Transform();



  public ViewModel(String fn) 
  // load and examine the model
  { 
    System.out.println("Loading model from " + fn);

    modelGroup = null;
    foundGroup = false;    // Group node not found yet
    isSceneRoot = true;    // assume it will be the root of the graph

    Object3D[] parts = null;
    try {
      parts = Loader.load(fn);
    }
    catch (Exception e) 
    { e.printStackTrace(); }

    examineObjects(parts, "", null, 0);

    adjustModelPosition();
  } // end of ViewModel()


  public Group getModel()
  {  return modelGroup;  }


  // ---------------- examine model ----------------

  private void examineObjects(Object3D[] parts, String section, 
                                       Object3D parent, int level)
  /* Examine each of the objects in the parts[] array. Print each
     object's class name and user ID. */
  {
    String name, subsection;
    for (int i=0; i < parts.length; i++) {
      name = parts[i].getClass().getName();
      subsection = section + (i+1) + ".";
      printlnIndent(subsection + " " + name + ", ID: " + 
                                  parts[i].getUserID(), level);
      examineNode(parts[i], name, parent, level);
      examineRefs(parts[i], subsection, level+1);
    }
  }  // end of examineObjects()


  private void examineNode(Object3D part, String name, 
                                         Object3D parent, int level)
  /* Store the top-level Group node in modelGroup.
     Examine Group, VertexBuffer, and Material nodes in more detail. */
  {
    if (name.endsWith("Group")) {
      storeGroup((Group) part, parent, level);
      examineGroup((Group) part, level);
    }
    else if (name.endsWith("VertexBuffer"))
      examineVertexBuffer((VertexBuffer) part, level);
    else if (name.endsWith("Material"))
      examineMaterial( (Material) part, level);
  }  // end of examineNode()


  private void storeGroup(Group g, Object3D parent, int level)
  /* Store the found Group node in modelGroup, and attempt to
     detach it from its parent if it isn't the root node of the 
     scene. */
  {
    if (!foundGroup) {   // if not already found a Group node
      modelGroup = g;
      System.out.println("modelGroup assigned Group with ID " + g.getUserID() +
                         "; at level " + level);
      foundGroup = true;
      if (level != 0) {   // the Group is not the root node
        isSceneRoot = false;
        if (parent instanceof Group) {
          Group gPar = (Group) parent;
          gPar.removeChild(modelGroup);
          System.out.println("Detached modelGroup from parent with ID " +
                                      gPar.getUserID() );
        }
        else {
          System.out.println("Could not detach modelGroup; set to null");
          modelGroup = null;   // since cannot attach it to our scene
        }
      }
    }
  }  // end of storeGroup()


  private void examineGroup(Group g, int level)
  // Print the the composite transform matrix for the Group
  {
    g.getCompositeTransform(modelTransform);
    modelTransform.get(trans);

    for(int i=0; i < 16; i= i+4)
      printlnIndent( round2dp(trans[i]) + " " + 
                     round2dp(trans[i+1]) + " " +
                     round2dp(trans[i+2]) + " " + 
                     round2dp(trans[i+3]), level);
  }  // end of examineGroup()


  private float round2dp(float num)
  // round num to 2 decimal places
  {  return ((int)((num+0.005)*100.0f))/100.0f;  }



  private void examineVertexBuffer(VertexBuffer vb, int level)
  /* Print information about the VertexBuffer's contents: the
     positions, colours, normals, and number of textures. */
  {
    float[] scaleBias = new float[4];
    VertexArray va = vb.getPositions(scaleBias);

    printlnIndent("Positions: " + (va != null) + 
                  ";  No. Coords: " + (vb.getVertexCount()/3), level);
    if (va != null)
      printlnIndent("Posn ID: " + va.getUserID() + 
                    ";  User Object: " + (va.getUserObject() != null), level);

    printlnIndent("Scale/Bias: " + round2dp(scaleBias[0]) + " " +
                                   round2dp(scaleBias[1]) + " " +
                                   round2dp(scaleBias[2]) + " " +
                                   round2dp(scaleBias[3]), level);
    int texCount = 0;
    while (true) {
      if (vb.getTexCoords(texCount, scaleBias) == null)
        break;
      texCount++;
    }

    printlnIndent("Colours: " + (vb.getColors() != null) + 
                  ";  Normals: " + (vb.getNormals() != null) +
                  ";  No. of textures: " + texCount, level);
  }  // end of examineVertexBuffer()


  private void examineMaterial(Material m, int level)
  /* Show colour and shininess info for the material. */
  {
    int ambColour = m.getColor(Material.AMBIENT);
    int diffColour = m.getColor(Material.DIFFUSE);
    int emisColour = m.getColor(Material.EMISSIVE);
    int specColour = m.getColor(Material.SPECULAR);

    printlnIndent("Colours: " +  ambColour + ", " + diffColour + ", " +
                                emisColour + ", " + specColour + ", " +
                                m.getShininess(), level);

    printlnIndent("Colours in Hex: " + Integer.toHexString(ambColour) + ", " + 
                                       Integer.toHexString(diffColour) + ", " +
                                       Integer.toHexString(emisColour) + ", " + 
                                       Integer.toHexString(specColour), level);

  }  // end of examineMaterial()



  private void examineRefs(Object3D obj, String section, int level)
  /* Examine all the references connected to obj. */
  {
    int numRefs = obj.getReferences(null);
    if (numRefs > 0) {
      Object3D[] parts = new Object3D[numRefs];
      obj.getReferences(parts);
      examineObjects(parts, section, obj, level);
    }
  }  // end of examineRefs()


  private void printlnIndent(String msg, int level)
  // indent msg by level*2 spaces
  {
    if (showExamination) {
      for(int i=0; i < level; i++)
        System.out.print("  ");   // 2 spaces
      System.out.println(msg);
    }
  }  // end of printlnIndent()


  private void adjustModelPosition()
  /* If modelGroup isn't the root node then it's probably been
     created in 3DS Max, and so is facing downwards. Rotate it to
     face forwards, and lift it up a little to be above the floor.
  */
  {
    if ((modelGroup != null) && (!isSceneRoot)) {
      System.out.println("Rotating the model -90 degrees around x-axis");
      System.out.println("Raising the model 0.1 units up y-axis");
      Group mGroup = modelGroup;
      modelGroup = new Group();
      modelGroup.setOrientation(-90.0f, 1.0f, 0, 0);  // rotate
      modelGroup.setTranslation(0.0f, 0.1f, 0.0f);    // raise
      modelGroup.addChild(mGroup);
    }
  }  // end of adjustModelPosition()


} // end of ViewModel class
